;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xa426ac9760c24634072b2dc9cc775ac9 . "./code.cg2js/cg-acache.fasl")
(#x65f478753a52be66a74f84006a53d9ce . "./code.cg2js/cg-base.fasl")
(#xd44cd5ff49da5fa14a129ad39dcba6d9 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x343ff3f2180cc455c17cc3fdb4bf47d1 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xb463cda7d93fe408858636c433243fbc . "./code.cg2js/cg-bitmap-stream.fasl")
(#xb232ce9b74f4b91b8b7cedf2ff08744a . "./code.cg2js/cg-button.fasl")
(#xd9fe75ef34c40d77de3e6248973a10ae . "./code.cg2js/cg-calendar.fasl")
(#x870bc89721d86866b9ffbc89e585f6bd . "./code.cg2js/cg-caret.fasl")
(#xd0be3a3e5d47114403317071c2d90332 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xed2b6dbb42e9e7a520ac913a5f12d013 . "./code.cg2js/cg-chart-widget.fasl")
(#x580a112e7683e2a2f27649901d649352 . "./code.cg2js/cg-check-box.fasl")
(#x93e8ae29a436eefe6146c84d03c37e34 . "./code.cg2js/cg-choice-list.fasl")
(#xb725fea0adbf8a4ed67710215f302dd4 . "./code.cg2js/cg-choose-printer.fasl")
(#x4e16831d8945ac1a88c6cd45f500347c . "./code.cg2js/cg-class-grid.fasl")
(#xc36789d6e465b9535330fd56cc23d245 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x5294b718ab60be6fb4866360fedf632f . "./code.cg2js/cg-class-support.fasl")
(#x82eae06550e679f1b350acafcaf05afa . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x9c6c6bcd8c8dabe5f9fc51112f97457a . "./code.cg2js/cg-clipboard-stack.fasl")
(#xef291d7def63eb8c32bd585e192e8910 . "./code.cg2js/cg-clipboard.fasl")
(#x61b0c2c99ca783d7a122034367be8558 . "./code.cg2js/cg-color-dialog.fasl")
(#x2e793d9717675108e2e5cb5693e7ca59 . "./code.cg2js/cg-combo-box.fasl")
(#x5c1b1395ebd39fbb61b21a9aab185416 . "./code.cg2js/cg-common-control.fasl")
(#xaee32a08d8e18403b9c2d2c636173c31 . "./code.cg2js/cg-comtab.fasl")
(#x3f85afd09352d57083f6b6f81e964cda . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xe541b76dc6135b12c13da2b72d6e8ba3 . "./code.cg2js/cg-curve.fasl")
(#x2750ba9e504a36427ab9f225cb14eac3 . "./code.cg2js/cg-dde-utils.fasl")
(#x65e9a08d22b97852ef8f45319d497143 . "./code.cg2js/cg-dev.fasl")
(#xf8790ac012858e30206897ee17c931d7 . "./code.cg2js/cg-dialog-item.fasl")
(#xee095e727f7bde4782b49fba6103ecf3 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xd1f280899b164bf53b212dea89a40eae . "./code.cg2js/cg-directory-dialog.fasl")
(#x70f5a6074f523c845ffbf0023c2dfdfb . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x0deb402a6c0e01422aa7b2c88084e943 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x0e25b0eaf91c6e7c7c04f524d2977c52 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x4c84c4f950165b683451bf68bc9a4b59 . "./code.cg2js/cg-drawable.fasl")
(#x4a6bca9523ad17ddafc23022fd522a30 . "./code.cg2js/cg-dropping-outline.fasl")
(#x592883a2da7c985e79b16c2cc198b98e . "./code.cg2js/cg-edit-in-place.fasl")
(#x8b947fc8ff6ba05269c0eb8c35348ff3 . "./code.cg2js/cg-editable-text.fasl")
(#xb5ac5b8f402659414522cdd6daf73146 . "./code.cg2js/cg-file-dialog.fasl")
(#xccf128a9a8afa42f255d510a621f08c1 . "./code.cg2js/cg-fill-texture.fasl")
(#x75a76676c3328c0c147e4012cf96b7ca . "./code.cg2js/cg-find-string-dialog.fasl")
(#x9ba3631f4868e3b23bff2c4f1502fc32 . "./code.cg2js/cg-font-dialog.fasl")
(#xf8f595fcd7b049d04b95408604d981bf . "./code.cg2js/cg-gesture-emulation.fasl")
(#x932fad300ca4b1923c6c221e25ec942c . "./code.cg2js/cg-get-pixmap.fasl")
(#xb28146183fd1f4c0bca7e5df26ec0a52 . "./code.cg2js/cg-get-position.fasl")
(#xed9372fdb22d84327274d586e7bb316e . "./code.cg2js/cg-graphics-context.fasl")
(#x7a06fc65ebc47e2634c4f38c2fc34d6e . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x73136a848e9ee137aac0a7ead32c35a4 . "./code.cg2js/cg-grid-widget.fasl")
(#xc663d30ba8177525d94e451eb418ff14 . "./code.cg2js/cg-group-box.fasl")
(#xe16ffb196a42c05aa728a2fab3c6d84a . "./code.cg2js/cg-header-control.fasl")
(#x17801820adb77d48a4645468bcf90efe . "./code.cg2js/cg-hotspot.fasl")
(#x8ce32ec16ae01a06b1feda41761fdae9 . "./code.cg2js/cg-html-dialog.fasl")
(#xaefd0d3be464359edbee0898e7b46350 . "./code.cg2js/cg-html-widget.fasl")
(#x874b59e32721fe58d43d752bd07c5e8e . "./code.cg2js/cg-icon-pixmap.fasl")
(#xe159bd081a2d3fa4024238a256b21caf . "./code.cg2js/cg-icon.fasl")
(#x844f7c3782a82cfbb165d644049d8daf . "./code.cg2js/cg-ie.fasl")
(#xc31d19564561a078831702fe150e7f9f . "./code.cg2js/cg-intersect-posbox.fasl")
(#x866943aae570883a7730ef9e99b1d2e5 . "./code.cg2js/cg-intersect.fasl")
(#x87953778710689c6e4c18ca6360da260 . "./code.cg2js/cg-item-list.fasl")
(#xfc817de7cf4b1315c2f53b0c718924ed . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xb401b5b1957fb89126b0b774f53638f0 . "./code.cg2js/cg-lamp.fasl")
(#x809354deb9a1421ea49d8dedaad70820 . "./code.cg2js/cg-layout.fasl")
(#x623dababea905ac7e9bae54278b9b7ca . "./code.cg2js/cg-lettered-menu.fasl")
(#xef1b444aec513987d1dc9b56d9599705 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xedfa7bf3b1fab2ac6115a344f0b031f4 . "./code.cg2js/cg-lisp-text.fasl")
(#xacba3601f57b225baf2e234913bf59f1 . "./code.cg2js/cg-lisp-widget.fasl")
(#xe25637880a168629ba6227f8bdbe747b . "./code.cg2js/cg-list-view.fasl")
(#x7e0f61f1b8f38c6cabca3328638dc445 . "./code.cg2js/cg-mci.fasl")
(#xa1e6a58a6915fba390812e43ec104041 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xce409fb3010de094f7dce8f6009e364c . "./code.cg2js/cg-menu.fasl")
(#x5dc0995eee3f42e42af321d6da493ed5 . "./code.cg2js/cg-message-dialog.fasl")
(#x3b722f171377d94c9821e5369492ed9f . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xf9ba3916557c356b202cec54eb584a7e . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xd4c2e85238826851eb9c0c3b0eb896b2 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x8804d805780222d3718973c7fc9c7bd2 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x43222ec9f5f70be7225bdc7b72367be1 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x15cf57692e847c6c802fadacb26daf6a . "./code.cg2js/cg-nodes.fasl")
(#x72a56598f772eb32039ed1d0a4f39946 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x407dd4a4ec38a8066917444757225e5f . "./code.cg2js/cg-object-editor.fasl")
(#xf335622e3c50666c5e7724e865ffdc5c . "./code.cg2js/cg-ocx.fasl")
(#xb761833d809f771372466d8348ee71a0 . "./code.cg2js/cg-os-widget.fasl")
(#xa9fadfe07ac2fb4b9cfa1f017a030c68 . "./code.cg2js/cg-os-window.fasl")
(#xf8a49327d118cbbb91434145bb01e9fd . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x0468a33873cef94ae2645e15a3ff59eb . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x9794fdf16bac0886e54f5ae5c43840b2 . "./code.cg2js/cg-outline.fasl")
(#x6145730a575a9f7d407a3e62bf6d44af . "./code.cg2js/cg-palette.fasl")
(#x0be5821861af195613a07f64d9d15bb0 . "./code.cg2js/cg-paren-matching.fasl")
(#x741d6aa83dad8efdaf2743beb4f8bcd0 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x1e1ccee52e42dff37899a7ef3c7d1556 . "./code.cg2js/cg-picture-widget.fasl")
(#xff579bd6932d3de1e91039a4dd94c727 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x6f412f8247b708e778765db615b253bc . "./code.cg2js/cg-pixmap-printing.fasl")
(#x2e4623f115833a4a3680601a633c8cdc . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x4778e7c42c2d4d6d90ff2843209ee626 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x64787730a301564ade31e95390d5e1c7 . "./code.cg2js/cg-pixmap.fasl")
(#xdfe5c846d5f58e24fdf487a68d953b42 . "./code.cg2js/cg-plot-widget.fasl")
(#x50ea3195def75b92b52088d5a68d783d . "./code.cg2js/cg-printing.fasl")
(#xa78ed614e38a24af4fa2ea8fd390b130 . "./code.cg2js/cg-progress-indicator.fasl")
(#xcd273fb3fce92597728ddd08ba488824 . "./code.cg2js/cg-project-window.fasl")
(#x232afdfc2922e0e4ee57677c8f9fbefc . "./code.cg2js/cg-property.fasl")
(#xe380d7018417463a97eff8804c1eefe9 . "./code.cg2js/cg-radio-button.fasl")
(#x35e43538470edd8925c5f32eddad38df . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xf165f9270f8115a6bad3088649277986 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xeea75ae1a55b20f45ac807e2f9fa7164 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x966c829ef7ed11c62338efff42f39736 . "./code.cg2js/cg-rich-edit.fasl")
(#x7d70beaf6949c30c438de5a657971a70 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x8f595950bf9fc1847942b8a9f570ed21 . "./code.cg2js/cg-scaling-stream.fasl")
(#x3b81ef260e7366fdf6a5984c66df35a4 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x72fa4fdfc6cbe9c0b948b3c801422561 . "./code.cg2js/cg-scroll-bar.fasl")
(#x0e2f1f41926ee4e0112a426949a917fb . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x4e7aa49bafaccc406cd9e681f6d88a23 . "./code.cg2js/cg-selected-object.fasl")
(#x34d208d650b1ccfb47ece9c1c4268982 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x889aa9d3146ca9b3f2667b48d60b964b . "./code.cg2js/cg-split-bar.fasl")
(#xd8c82472ba7611c8e11b07f2ef4e716a . "./code.cg2js/cg-static-text.fasl")
(#x9ea920d80dec0ed4447596c9050ba931 . "./code.cg2js/cg-status-bar.fasl")
(#xcc54c91e09cc58a9131f6f30382b686a . "./code.cg2js/cg-string-dialog.fasl")
(#xdb157b660ea898be8cb9c59b5bcbb4ff . "./code.cg2js/cg-tab-control.fasl")
(#x07b7fbde846b6b4f07165256747debbb . "./code.cg2js/cg-template-string.fasl")
(#xa53b31a52c0f2ec32d4729bca16c55b3 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x839e27306729bb48456b2c75a62bddf8 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xaf36d4b7ddee9bf704a4005cbf2fcd59 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x292fca4b6408be9d62f106d3c04e7af1 . "./code.cg2js/cg-text-or-combo.fasl")
(#x2a3984dd230973a64da70f3ae2791c18 . "./code.cg2js/cg-text-widget.fasl")
(#x1f21767a2bfe979335aa6395b099094a . "./code.cg2js/cg-timer.fasl")
(#xec4c77979f87d04360b08a0a08af280c . "./code.cg2js/cg-toggling-widget.fasl")
(#x25bf0d934da5b343e89210550e880f79 . "./code.cg2js/cg-toolbar.fasl")
(#x7ef7b173541b21e8478801e3e4e3202b . "./code.cg2js/cg-tooltip.fasl")
(#x61be120cf9d84df7488521fc1892d97d . "./code.cg2js/cg-touch.fasl")
(#xbf38568c57ec90d57d12c2d42f35c030 . "./code.cg2js/cg-trackbar.fasl")
(#x5d6b98b6ed3878da9edfca136efaf000 . "./code.cg2js/cg-tray.fasl")
(#xa90ddd71e5044ae3d524193e99e16281 . "./code.cg2js/cg-up-down-control.fasl")
(#xa6158ef8305c6a899dd0144d7ebf9db6 . "./code.cg2js/cg-utility-dialog.fasl")
(#xac615f1b9719031e5f1c53356a121ed1 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x388b5c86e72b5a0485225360a4c76f56 . "./code.cg2js/cg-web-browser.fasl")
(#xa19be217d69ec3464c6e0c83b680e5a3 . "./code.cg2js/cg-wrap-string.fasl")
(#x0559c29c4c110ea542d8f66503734469 . "./code.cg2js/cg-yes-no-list.fasl")
(#x0c39b77cd9a428fafd921c432916f759 . "./code.cg2js/cg-yes-no-string.fasl")
(#x6f218d1430b10dde3bf775b2816bd3bc . "./code.cg2js/cg.fasl")
(#x1f9c9cf6cb46e042e7c3e29ae1b32e79 . "./code.idejs/ide-acache.fasl")
(#xb97500a224f49d57a06506a91622fa51 . "./code.idejs/ide-apropos.fasl")
(#xe7a93e1047064340d4994cfc06d53843 . "./code.idejs/ide-base.fasl")
(#x7bcd42e12e1be6fecd4a01fcc63ceb52 . "./code.idejs/ide-class-browser.fasl")
(#x65c17a3f53ec79964a9d601358b64588 . "./code.idejs/ide-class-grapher.fasl")
(#xaccab05b36b9797ac773e0c0175ff414 . "./code.idejs/ide-clipboard.fasl")
(#x55b60aec923caa56e4a7689ec787ca00 . "./code.idejs/ide-dev.fasl")
(#x3b3e7d63cb99fa16d3b9779acb684802 . "./code.idejs/ide-editor.fasl")
(#x2a048fc8a7716ad3ed21f1609b35d000 . "./code.idejs/ide-find-definitions.fasl")
(#xe0aa358b0809e1f63788607027b37744 . "./code.idejs/ide-find-in-files.fasl")
(#xa4da420f0e9bce7276785ba18518b7a7 . "./code.idejs/ide-form.fasl")
(#xa445659bc0a135c7373166275fa84d5a . "./code.idejs/ide-grapher.fasl")
(#x355c718c47d20978c2ea7e758af59b57 . "./code.idejs/ide-inspector.fasl")
(#xb286b2cb973a785254d7c15d1878e380 . "./code.idejs/ide-intro.fasl")
(#x985f4d1293b14663429164eb3684d0cb . "./code.idejs/ide-keyshort.fasl")
(#xd754a3469fe08ec880f4d0973776e596 . "./code.idejs/ide-navigator.fasl")
(#x67201b482c3584f5dd448eb7215df0e4 . "./code.idejs/ide-package-list.fasl")
(#x27824113669cbd8db0fda5380f374f2f . "./code.idejs/ide-patch-dialog.fasl")
(#x4dbe19a6a241bfde4902f7de34764330 . "./code.idejs/ide-process-browser.fasl")
(#xa7a968bd6fe7ffd89a469b3add4cf401 . "./code.idejs/ide-project.fasl")
(#x7507f538ec10c53c696973fb7a92226f . "./code.idejs/ide-trace.fasl")
(#xdbef726e716b9a1f6fa3054dd7d22247 . "./code.idejs/ide-window-list.fasl")
(#xbedb526ff96569388eaea80231b1cec1 . "./code.idejs/ide.fasl")
(#xc2912bce53894841a1e9ff8f0fd00d64 . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
